#!/usr/bin/env python

"""
This script computes the following binary metrics
  * tp
  * fp
  * fn
  * precision
  * recall
  * Jaccard index

Tests can be executed by running
> python binary_metrics.py
"""


class RankMetrics:
    """Class for saving metrics per rank
    """

    def __init__(self, rank):
        self.__rank = rank

    @property
    def rank(self):
        return self.__rank

    @property
    def tp(self):
        return self.__tp

    @property
    def fn(self):
        return self.__fn

    @property
    def fp(self):
        return self.__fp

    @property
    def precision(self):
        return self.__precision

    @property
    def recall(self):
        return self.__recall

    @property
    def jaccard(self):
        return self.__jaccard

    @property
    def f1(self):
        return self.__f1

    @property
    def otus(self):
        return self.__otus

    @rank.setter
    def rank(self, rank):
        self.__rank = rank

    @fn.setter
    def fn(self, fn):
        self.__fn = fn

    @tp.setter
    def tp(self, tp):
        self.__tp = tp

    @fp.setter
    def fp(self, fp):
        self.__fp = fp

    @precision.setter
    def precision(self, precision):
        self.__precision = precision

    @recall.setter
    def recall(self, recall):
        self.__recall = recall

    @jaccard.setter
    def jaccard(self, jaccard):
        self.__jaccard = jaccard

    @f1.setter
    def f1(self, f1):
        self.__f1 = f1

    @otus.setter
    def otus(self, otus):
        self.__otus = otus

    def get_ordered_dict(self):
        from collections import OrderedDict
        return OrderedDict(sorted(self.__dict__.items()))

    def get_dict(self):
        return self.__dict__

    def get_pretty_dict(self):
        return {metric.split("_")[3]: value for metric, value in self.__dict__.items()}


def __get_existing_taxa(rank):
    """ Return set of taxids with abundance > 0

    >>> __get_existing_taxa(query_rank)
    [123]

    :param rank: Set of taxids of specific rank
    :return: list of taxids
    """
    return list(k for k, v in rank.items() if v > 0)


def __get_non_existing_taxa(rank_query, rank_truth):
    """Return set of taxids with abundance <= 0
    >>> __get_non_existing_taxa(query_rank, truth_rank)
    []
    >>> __get_non_existing_taxa(query_rank2, truth_rank)
    [122]

    :param rank: Set of taxids of specific rank
    :return: list of taxids
    """
    rank_truth_taxids = set(k for k, v in rank_truth.items() if v > 0)
    rank_query_taxids = set(k for k, v in rank_query.items() if v > 0)
    return list(rank_query_taxids.difference(rank_truth_taxids))


def __get_tp(rank_query, rank_truth):
    """ Returns true positive
    >>> __get_tp(test_query_rank, test_truth_rank)
    1

    """
    rank_query_taxids = __get_existing_taxa(rank_query)
    rank_truth_taxids = __get_existing_taxa(rank_truth)
    return len(list(set(rank_query_taxids).intersection(rank_truth_taxids)))


def __get_fp(rank_query, rank_truth):
    """ Returns false positive
    >>> __get_fp(test_query_rank, test_truth_rank)
    0

    """

    rank_query_taxids = __get_existing_taxa(rank_query)
    rank_truth_taxids = __get_non_existing_taxa(rank_query, rank_truth)
    return len(list(set(rank_query_taxids).intersection(rank_truth_taxids)))


def __get_fn(rank_query, rank_truth):
    """ Returns false negative
    >>> __get_fn(test_query_rank, test_truth_rank)
    0

    """
    rank_query_taxids = __get_non_existing_taxa(rank_truth, rank_query)
    rank_truth_taxids = __get_existing_taxa(rank_truth)
    return len(list(set(rank_query_taxids).intersection(rank_truth_taxids)))


def precision(tp, fp):
    if (tp + fp) == 0:
        return 0
    return tp / (tp + fp)


def recall(tp, fn):
    if (tp + fn) == 0:
        return 0
    return tp / (tp + fn)


def jaccard_index(rank_query, rank_truth):
    """ Returns the Jaccard index
    >>> jaccard_index(test_query_rank, test_truth_rank)
    1.0

    """
    rank_query = __get_existing_taxa(rank_query)
    rank_truth = __get_existing_taxa(rank_truth)
    intersection = len(list(set(rank_query).intersection(rank_truth)))
    union = len(list(set(rank_query).union(rank_truth)))
    if union > 0:
        return intersection / union
    else:
        return .0


def f1_score(this_precision, this_recall):
    """ Returns f1 score
    >>> f1_score(rank_metrics.precision, rank_metrics.recall)
    1.0

    """
    if this_precision + this_recall > 0:
        return 2 * this_precision * this_recall / (this_precision + this_recall)
    else:
        return None


def compute_rank_metrics(rank_query, rank_truth, rank):
    """ Returns metrics for one rank
    >>> compute_rank_metrics(test_query_rank, test_truth_rank, "species").get_ordered_dict()
    OrderedDict([('_RankMetrics__f1', 1.0), ('_RankMetrics__fn', 0), ('_RankMetrics__fp', 0), ('_RankMetrics__jaccard', 1.0), ('_RankMetrics__otus', 1), ('_RankMetrics__precision', 1.0), ('_RankMetrics__rank', 'species'), ('_RankMetrics__recall', 1.0), ('_RankMetrics__tp', 1)])

    """
    tp = __get_tp(rank_query, rank_truth)
    fn = __get_fn(rank_query, rank_truth)
    fp = __get_fp(rank_query, rank_truth)
    rank_metrics = RankMetrics(rank)
    rank_metrics.tp = tp
    rank_metrics.fn = fn
    rank_metrics.fp = fp
    rank_metrics.precision = precision(tp, fp)
    rank_metrics.recall = recall(tp, fn)
    rank_metrics.f1 = f1_score(rank_metrics.precision, rank_metrics.recall)
    rank_metrics.jaccard = jaccard_index(rank_query, rank_truth)
    rank_metrics.otus = tp + fp
    return rank_metrics


def compute_tree_metrics(query, truth):
    """ Return metrics for tree
    >>> compute_tree_metrics(query_tree, truth_tree)["species"].get_ordered_dict()
    OrderedDict([('_RankMetrics__f1', 0.5), ('_RankMetrics__fn', 1), ('_RankMetrics__fp', 3), ('_RankMetrics__jaccard', 0.3333333333333333), ('_RankMetrics__otus', 5), ('_RankMetrics__precision', 0.4), ('_RankMetrics__rank', 'species'), ('_RankMetrics__recall', 0.6666666666666666), ('_RankMetrics__tp', 2)])
    """

    def check_for_rank(query, rank):
        """Make sure that empty level is produced
        """
        if rank in query:
            return query[rank]
        else:
            return {}

    return {rank: compute_rank_metrics(check_for_rank(query, rank), taxids, rank) for rank, taxids in truth.items()}


def print_all_metrics(tree_metrics, path):
    import csv

    def get_header(tree):
        not_empty_ranks = [metrics.get_pretty_dict() for rank, metrics in tree.items() if
                           bool(metrics.get_pretty_dict())]
        if len(not_empty_ranks) > 0:
            return not_empty_ranks[0].keys()
        else:
            return []

    with open(path, 'w') as metrcs_out:
        writer = csv.DictWriter(metrcs_out, fieldnames=get_header(tree_metrics),
                                restval='ignore', delimiter='\t')
        writer.writeheader()
        {rank: writer.writerow(metrics.get_pretty_dict()) for rank, metrics in tree_metrics.items()}


if __name__ == "__main__":
    import doctest

    test_query_rank = dict()
    test_query_rank[123] = 0.1
    test_query_rank[1232] = 0.0

    test_truth_rank = dict()
    test_truth_rank[123] = 0.1
    test_truth_rank[1232] = 0.0

    test_query_rank2 = dict()
    test_query_rank2[123] = 0.1
    test_query_rank2[122] = 4.0

    test_truth_tree = dict()
    test_truth_tree["species"] = {"A": 0.5,
                                  "B": 0.3,
                                  "E": 0.7}

    test_query_tree = dict()
    test_query_tree["species"] = {"A": 0.5,
                                  "B": 0.9,
                                  "C": 0.4,
                                  "D": 0.2,
                                  "F": 0.2}

    tree_metrics = dict()
    rank_metrics = RankMetrics("species")
    rank_metrics.tp = 0
    rank_metrics.fn = 0
    rank_metrics.fp = 1
    rank_metrics.precision = 1
    rank_metrics.recall = 1
    rank_metrics.jaccard = 1
    rank_metrics.f1 = 1
    tree_metrics["species"] = rank_metrics

    doctest.testmod(extraglobs={'query_rank': test_query_rank,
                                'query_rank2': test_query_rank2,
                                'truth_rank': test_truth_rank,
                                'truth_tree': test_truth_tree,
                                'tree_metrics': tree_metrics,
                                'query_tree': test_query_tree})



from utils import constants as c
import pandas as pd

from bokeh.embed import file_html
from bokeh.resources import CDN
from bokeh.plotting import figure, output_file, show
from bokeh.models import ColumnDataSource
from bokeh.layouts import widgetbox, layout, column, row
from bokeh.models.widgets import DataTable, DateFormatter, TableColumn, TextInput, Slider, Div, RadioGroup
from bokeh.models import (ColorBar,
                          Text,
                          BasicTicker,
                          HoverTool,
                          FuncTickFormatter,
                          DataTable,
                          widgets,
                          CustomJS)


def print_by_rank(labels, pd_metrics):
    for rank in c.ALL_RANKS:
        # define ordering of rows, which is given my order of tool labels
        order_rows = labels
        # define ordering of columns, hard coded
        order_columns = [c.UNIFRAC, c.UNW_UNIFRAC, c.L1NORM, c.RECALL, c.PRECISION, c.F1_SCORE, c.TP, c.FP, c.FN, c.JACCARD, c.SHANNON_DIVERSITY, c.SHANNON_EQUIT, c.BRAY_CURTIS]
        # subset to those information that either belong to the given rank or are rank independent, i.e. are unifrac values
        table = pd_metrics[(pd_metrics['rank'] == rank) | (pd_metrics['metric'].isin([c.UNIFRAC, c.UNW_UNIFRAC]))]
        # reformat the table with a pivot_table
        table = table.pivot_table(index=['tool', 'sample'], columns='metric', values='value')
        # select only tools in labels and get rid of gold standard
        table = table.loc[pd.IndexSlice[order_rows,:], order_columns]
        # define categorical column for ordering rows by tools
        table['tool_cat'] = pd.Categorical(table.index.get_level_values('tool'), categories=order_rows, ordered=True)
        # order table
        table = table.sort_values('tool_cat')
        table = table.loc[:, order_columns]
        # replace np.NaN with string "na" and write resulting table into a file
        return table.fillna('na')
        #table.fillna('na').to_csv(os.path.join(output_dir, "by_rank", rank + ".tsv"), sep='\t')


def create_html(pd_rankings, pd_metrics, labels):
    all_metrics = 'Sum of scores'
    df_list = []
    df_list_unsorted_pos = []
    metrics_list = []
    for metric, g in pd_rankings.groupby(level=0):
        metrics_list.append(metric)
        df = g.reset_index().sort_values('position')
        df2 = g.reset_index()
        df_list.append(pd.DataFrame({metric: df['tool'].tolist(), 'score' + metric: df['position'].tolist()}))
        df_list_unsorted_pos.append(pd.DataFrame({metric: df2['tool'].tolist(), 'score' + metric: df2['position'].tolist()}))

    df_sum = pd_rankings.groupby(['tool'])['position'].sum().reset_index().sort_values('position')
    df_sum_unsorted_pos = pd_rankings.groupby(['tool'])['position'].sum().reset_index()
    df_list.append(
        pd.DataFrame({all_metrics: df_sum['tool'].tolist(), 'score' + all_metrics: df_sum['position'].tolist()}))
    df_list_unsorted_pos.append(
        pd.DataFrame({all_metrics: df_sum_unsorted_pos['tool'].tolist(), 'score' + all_metrics: df_sum_unsorted_pos['position'].tolist()}))

    pd_show = pd.concat(df_list, axis=1)
    pd_show_unsorted_pos = pd.concat(df_list_unsorted_pos, axis=1)

    table_source = ColumnDataSource(pd_show)

    columns = [
        TableColumn(field=all_metrics, title=all_metrics, sortable=False),
        TableColumn(field='score' + all_metrics, title='', width=50),
        TableColumn(field=c.RECALL, title=c.RECALL, sortable=False),
        TableColumn(field='score' + c.RECALL, title='', width=50),
        TableColumn(field=c.PRECISION, title=c.PRECISION, sortable=False),
        TableColumn(field='score' + c.PRECISION, title='', width=50),
        TableColumn(field=c.L1NORM, title=c.L1NORM, sortable=False),
        TableColumn(field='score' + c.L1NORM, title='', width=50),
        TableColumn(field=c.UNIFRAC, title=c.UNIFRAC, sortable=False),
        TableColumn(field='score' + c.UNIFRAC, title='', width=50),
    ]
    data_table = DataTable(source=table_source, columns=columns, width=800, height=25 + len(pd_show) * 25)

    top = [float(x) for x in pd_show_unsorted_pos['score' + all_metrics]]
    source = ColumnDataSource(data=dict(x=pd_show_unsorted_pos[all_metrics].tolist(),
                                        top=top,
                                        recall=pd_show_unsorted_pos['score' + c.RECALL],
                                        precision=pd_show_unsorted_pos['score' + c.PRECISION],
                                        l1norm=pd_show_unsorted_pos['score' + c.L1NORM],
                                        unifrac=pd_show_unsorted_pos['score' + c.UNIFRAC]))

    callback = CustomJS(args=dict(source=source), code="""
        var data = source.data;
        var wrecall = weight_recall.value;
        var wprecision = weight_precision.value;
        var wl1norm = weight_l1norm.value;
        var wunifrac = weight_unifrac.value;
        topx = data['top'];
        recall = data['recall'];
        precision = data['precision'];
        l1norm = data['l1norm'];
        unifrac = data['unifrac'];
        
        
        for (i = 0; i < topx.length; i++) {
            topx[i] = recall[i] * wrecall + precision[i] * wprecision + l1norm[i] * wl1norm + unifrac[i] * wunifrac;
        }
        
        source.change.emit();
    """)


    #weight_recall = TextInput(title=c.RECALL + " weight", value='1.0', callback=callback)
    weight_recall = Slider(start=0, end=10, value=1, step=.1, title=c.RECALL + " weight", callback=callback)
    callback.args["weight_recall"] = weight_recall

    weight_precision = Slider(start=0, end=10, value=1, step=.1, title=c.PRECISION + " weight", callback=callback)
    callback.args["weight_precision"] = weight_precision

    weight_l1norm = Slider(start=0, end=10, value=1, step=.1, title=c.L1NORM + " weight", callback=callback)
    callback.args["weight_l1norm"] = weight_l1norm

    weight_unifrac = Slider(start=0, end=10, value=1, step=.1, title=c.UNIFRAC + " weight", callback=callback)
    callback.args["weight_unifrac"] = weight_unifrac


    # p.vbar(x=metrics_list, width=0.5, bottom=0, top=pd_show['score'+all_metrics].tolist(), color="firebrick")
    p = figure(x_range=pd_show_unsorted_pos[all_metrics].tolist(), plot_width=800, plot_height=400, title=all_metrics + " - lower value is better")
    # topx[:] = [0 for x in pd_show['score' + all_metrics]]
    # p.vbar(x=pd_show[all_metrics].tolist(), width=0.5, bottom=0, top="top", color="firebrick", source=source)
    # source = ColumnDataSource(data=dict(top=[]))
    p.vbar(x='x', top='top', source=source, width=0.5, bottom=0, color="firebrick")
    #p.vbar(x=pd_show[all_metrics].tolist(), width=0.5, bottom=0, color="firebrick", top=top)

    #table_str = "<table><tr><td>a</td><td>b</td></tr></table>"
    #table_str = pd_metrics.to_html()

    radio_group = RadioGroup(labels=["Option 1", "Option 2", "Option 3"], active=0)
    table_str = print_by_rank(labels, pd_metrics).T.to_html()

    mytable1 = Div(text="""<div>{}</div>""".format(table_str))
    #mytable2 = Div(children=mytable1, text="my text2")

    l = layout([radio_group, mytable1, data_table,
                weight_recall,
                weight_precision,
                weight_l1norm,
                weight_unifrac,
                p])

    # l = row(
    #     widgetbox(weight_recall),
    #     column(p)
    # )

    html = file_html(l, CDN, "OPAL")  # sizing_mode='scale_width'
    file = open('test.html', 'w+')
    file.write(html)
    file.close()
    # output_file("test.html", title="OPAL")
    # show(l)



    # for x in pd_show['score' + all_metrics]:
    #     topx.append(float(x))
    # topx[:] = [float(x) for x in pd_show['score' + all_metrics]]

    # sizing_mode = 'fixed'  # 'scale_width' also looks nice with this example
    #
    # inputs = widgetbox(*controls, sizing_mode=sizing_mode)
    # l = layout([
    #     [inputs, p],
    # ], sizing_mode=sizing_mode)







# This is an example of how to use the script
from utils import ProfilingTools as PF
from utils import EMDUnifrac as EMDU
import numpy as np
import argparse
import copy
import sys
import os


def compute_unifrac(pf1, pf2):
    P1 = copy.deepcopy(pf1)
    P2 = copy.deepcopy(pf2)
    (Tint, lint, nodes_in_order, nodes_to_index, P, Q) = P1.make_unifrac_input_and_normalize(P2)
    (weighted, _) = EMDU.EMDUnifrac_weighted(Tint, lint, nodes_in_order, P, Q)
    (unweighted, _) = EMDU.EMDUnifrac_unweighted(Tint, lint, nodes_in_order, P, Q)
    return weighted, unweighted


def weighted_unifrac(pf1, pf2):
    P1 = copy.deepcopy(pf1)
    P2 = copy.deepcopy(pf2)
    (Tint, lint, nodes_in_order, nodes_to_index, P, Q) = P1.make_unifrac_input_and_normalize(P2)
    (val, _) = EMDU.EMDUnifrac_weighted(Tint, lint, nodes_in_order, P, Q)
    return val


def unweighted_unifrac(pf1, pf2):
    P1 = copy.deepcopy(pf1)
    P2 = copy.deepcopy(pf2)
    (Tint, lint, nodes_in_order, nodes_to_index, P, Q) = P1.make_unifrac_input_and_normalize(P2)
    (val, _) = EMDU.EMDUnifrac_unweighted(Tint, lint, nodes_in_order, P, Q)
    return val


def print_list_unifrac(unifrac_list, labels, stream=sys.stdout):
    labels_iterator = iter(labels)
    stream.write("tool\tweighted\tunweighted\n")
    for unifrac in unifrac_list:
        stream.write("{}\t".format(next(labels_iterator)))
        stream.write("\t".join([format(unifrac[0], '.3f'), format(unifrac[1], '.3f')]))
        stream.write("\n")


def read_params(args):
	parser = argparse.ArgumentParser(description='')
	arg = parser.add_argument
	arg('--input', metavar='files_file', type=str, required=True,
					default=None, help="File of CAMI profile files to use")
	arg('--output', metavar='output_file', required=True, default=None, type=str,
					help="Output file (you should have this end in .csv as it is a matrix)")
	arg('--threshold', metavar='threshold', required=True, default=None, type=float,
					help="Value to threshold profiles to before computing EMDUnifrac. "
					"NOTE THIS VALUE IS IN PERCENTAGES so if you want 1% use 1")
	return vars(parser.parse_args())


if __name__ == '__main__':
	par = read_params(sys.argv)
	files_file = par['input']
	output_file = par['output']
	threshold = par['threshold']

	# Get all the profile file names
	files = []
	fid = open(files_file, 'r')
	for line in fid.readlines():
		files.append(line.strip())
	fid.close()

	# Import all the profiles
	profiles = []
	for file_name in files:
		profiles.append(PF.Profile(input_file_name=file_name))

	# Threshold all the profiles
	for profile in profiles:
		profile.threshold(threshold=threshold)

	# Compute EMDUnifrac
	D = np.zeros((len(profiles), len(profiles)))
	for i in range(len(profiles)):
		for j in range(i+1, len(profiles)):
			(wu, uu) = compute_unifrac(profiles[i], profiles[j])
			D[i, j] = wu
			D[j, i] = uu

	# Save results in tsv
	np.savetxt(output_file, D, delimiter='\t', newline='\n')



#!/usr/bin/env python

import os
import pandas as pd
import numpy as np
import itertools
import math
import copy
from collections import defaultdict
from collections import OrderedDict
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.lines as mlines
from matplotlib.ticker import MaxNLocator
from mpl_toolkits.mplot3d import Axes3D
import braycurtis as bc
from utils import spider_plot_functions as spl
from utils import constants as c
from utils import load_data
import scipy.special


def create_colors_list():
    colors_list = []
    for color in plt.cm.tab10(np.linspace(0, 1, 10))[:-1]:
        colors_list.append(tuple(color))
    colors_list.append("black")
    for color in plt.cm.Set2(np.linspace(0, 1, 8)):
        colors_list.append(tuple(color))
    for color in plt.cm.Set3(np.linspace(0, 1, 12)):
        colors_list.append(tuple(color))
    return colors_list


def plot_rarefaction_curves(gs_samples_list, output_dir, log_scale=False):
    colors_list = create_colors_list()
    fig, axs = plt.subplots(figsize=(6, 5))

    # turn on grid
    axs.grid(which='major', linestyle='-', linewidth='0.5', color='lightgrey')

    axs.xaxis.set_major_locator(MaxNLocator(integer=True))

    axs.set_xlabel('Number of samples')
    if log_scale:
        axs.set_ylabel('log$_{10}$(number of taxa)')
    else:
        axs.set_ylabel('Number of taxa')

    x = list(range(1, len(gs_samples_list) + 1))
    y_rank_to_rar = OrderedDict((rank, []) for rank in c.ALL_RANKS)
    y_rank_to_acc = OrderedDict((rank, []) for rank in c.ALL_RANKS)

    for i, rank in enumerate(c.ALL_RANKS):
        tax_id_to_num_occurrences = {}

        # compute accumulation curves
        num_ids = 0
        for sample in gs_samples_list:
            sample_id, sample_metadata, profile = sample
            for prediction in profile:
                if prediction.rank == rank and prediction.percentage > .0:
                    if prediction.taxid in tax_id_to_num_occurrences:
                        tax_id_to_num_occurrences[prediction.taxid] += 1
                    else:
                        tax_id_to_num_occurrences[prediction.taxid] = 1
            num_ids = len(tax_id_to_num_occurrences)
            if log_scale:
                y_rank_to_rar[rank].append(math.log10(num_ids) if num_ids > 0 else 0)
            else:
                y_rank_to_rar[rank].append(num_ids)

        # compute rarefaction curves
        num_samples = len(gs_samples_list)
        for j, sample in enumerate(gs_samples_list, 1):
            sum_over_taxa = 0
            for tax_id in tax_id_to_num_occurrences:
                sum_over_taxa += scipy.special.binom(num_samples - tax_id_to_num_occurrences[tax_id], j)
            if log_scale:
                value = num_ids - (scipy.special.binom(num_samples, j) ** (-1) * sum_over_taxa)
                y_rank_to_acc[rank].append(math.log10(value) if value > 0 else 0)
            else:
                y_rank_to_acc[rank].append(num_ids - (scipy.special.binom(num_samples, j) ** (-1) * sum_over_taxa))

    for i, rank in enumerate(c.ALL_RANKS):
        axs.plot(x, y_rank_to_acc[rank], color=colors_list[i])
    for i, rank in enumerate(c.ALL_RANKS):
        axs.plot(x, y_rank_to_rar[rank], linestyle=':', color=colors_list[i])

    if log_scale:
        file_name = 'rarefaction_curves_log_scale'
    else:
        file_name = 'rarefaction_curves'

    lgd = plt.legend(c.ALL_RANKS, bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0., handlelength=2, frameon=False)
    fig.savefig(os.path.join(output_dir, 'gold_standard', file_name + '.pdf'), dpi=100, format='pdf', bbox_extra_artists=(lgd,), bbox_inches='tight')
    fig.savefig(os.path.join(output_dir, 'gold_standard', file_name + '.png'), dpi=100, format='png', bbox_extra_artists=(lgd,), bbox_inches='tight')
    plt.close(fig)
    return [file_name]


def plot_samples_hist(gs_samples_list, sample_ids_list, output_dir):
    plots_list = []
    for rank in c.ALL_RANKS:
        df_gs = pd.DataFrame()
        for sample in gs_samples_list:
            sample_id, sample_metadata, profile = sample
            gs_sampleid_to_rank_to_taxid_to_percentage = {sample_id: load_data.get_rank_to_taxid_to_percentage(profile, rank)}
            if len(gs_sampleid_to_rank_to_taxid_to_percentage[sample_id]) == 0:
                continue

            # add taxa names
            tax_id_to_name = load_data.get_taxa_names(profile)
            gs_sampleid_to_rank_to_taxidandname_to_percentage = copy.deepcopy(gs_sampleid_to_rank_to_taxid_to_percentage)
            for tax_id, percentage in gs_sampleid_to_rank_to_taxid_to_percentage[sample_id][rank].items():
                gs_sampleid_to_rank_to_taxidandname_to_percentage[sample_id][rank][tax_id_to_name[tax_id] + ' ' + tax_id] = percentage
                del gs_sampleid_to_rank_to_taxidandname_to_percentage[sample_id][rank][tax_id]

            df_gs_sample = pd.DataFrame.from_dict(gs_sampleid_to_rank_to_taxidandname_to_percentage[sample_id][rank], orient='index')
            df_gs_sample.index.name = 'taxid'
            df_gs_sample['Sample'] = sample_id
            df_gs_sample.rename(columns={0: 'value'}, inplace=True)
            df_gs_sample.set_index('Sample', append=True, inplace=True)
            df_gs = pd.concat([df_gs, df_gs_sample])
        if df_gs.empty:
            continue

        df_gs = df_gs.pivot_table(index=['taxid'], columns='Sample', values='value').T

        df_gs = df_gs.loc[sample_ids_list]

        fig, axs = plt.subplots(figsize=(6, 5))
        axs.tick_params(axis='x',
                        which='both',
                        bottom='off',
                        top='off',
                        labelbottom='off')
        axs.set_ylabel('Proportions [%]')
        axs.set_title('Rank: ' + rank)
        fig_name = os.path.join('gold_standard', rank)
        fig = df_gs.plot(kind='bar', ax=axs, stacked=True, legend=False, colormap=plt.get_cmap('gist_rainbow')).get_figure()
        fig.savefig(os.path.join(output_dir, fig_name + '.pdf'), dpi=100, format='pdf', bbox_inches='tight')
        fig.savefig(os.path.join(output_dir, fig_name + '.png'), dpi=100, format='png', bbox_inches='tight')

        legend, labels = axs.get_legend_handles_labels()
        fig2, axs2 = plt.subplots(figsize=(1, 1))
        axs2.xaxis.set_visible(False)
        axs2.yaxis.set_visible(False)
        for v in axs2.spines.values():
            v.set_visible(False)

        ncol = int(math.ceil(df_gs.shape[1]/35))
        axs2.legend(legend, labels, loc='center', frameon=True, ncol=ncol, handletextpad=0, columnspacing=1.0, borderaxespad=0., fontsize=9)
        plt.tight_layout()
        fig2.savefig(os.path.join(output_dir, fig_name + '_legend.png'), dpi=100, format='png', bbox_inches='tight')

        plt.close(fig)
        plt.close(fig2)
        plots_list.append(fig_name)
    return plots_list


def do_scatter_plot(profile_values, gs_values, output_dir, rank, label):
    fig, axs = plt.subplots(figsize=(6, 5))

    # force axes to be from 0 to 100%
    axs.set_xlim([0.0, 1.0])
    axs.set_ylim([0.0, 1.0])

    # turn on grid
    axs.grid(which='major', linestyle='-', linewidth='0.5', color='lightgrey')

    axs.scatter(profile_values, gs_values, alpha=0.5)
    axs.set_xlabel(label)
    axs.set_ylabel('Gold standard')
    axs.set_title(c.BRAY_CURTIS + ' - Rank: ' + rank)

    fig_name = os.path.join("by_tool", label, 'beta_diversity_bc_' + rank)
    fig.savefig(os.path.join(output_dir, fig_name + '.pdf'), dpi=100, format='pdf', bbox_inches='tight')
    fig.savefig(os.path.join(output_dir, fig_name + '.png'), dpi=100, format='png', bbox_inches='tight')
    plt.close(fig)
    return [fig_name]


def plot_beta_diversity(gs_samples_list, profiles_list_to_samples_list, sample_ids_list, labels, output_dir):
    if len(sample_ids_list) < 2:
        return []

    gs_sampleid_to_rank_to_taxid_to_percentage = {}
    for sample in gs_samples_list:
        sample_id, sample_metadata, profile = sample
        gs_sampleid_to_rank_to_taxid_to_percentage[sample_id] = load_data.get_rank_to_taxid_to_percentage(profile)

    # Gold standard
    sample_ids_combinations = list(itertools.combinations(sample_ids_list, 2))
    gs_sample12_to_rank_to_braycurtis = {}
    # Compute Bray-Curtis for all combinations of samples
    for sample1, sample2 in sample_ids_combinations:
        gs_sample12_to_rank_to_braycurtis[(sample1, sample2)] = bc.braycurtis(gs_sampleid_to_rank_to_taxid_to_percentage[sample1], gs_sampleid_to_rank_to_taxid_to_percentage[sample2])
    # Sort by rank
    gs_rank_to_sample12_to_braycurtis = defaultdict(dict)
    for sample12, rank_to_braycurtis in gs_sample12_to_rank_to_braycurtis.items():
        for rank, value in rank_to_braycurtis.items():
            gs_rank_to_sample12_to_braycurtis[rank][sample12] = value

    plots_list = []

    # Assessed profiles
    profile_sampleid_to_rank_to_taxid_to_percentage = {}
    for profile, label in zip(profiles_list_to_samples_list, labels):
        profile_sampleid_to_rank_to_taxid_to_percentage[label] = {}
        for sample in profile:
            sample_id, sample_metadata, profile = sample
            if not sample_id in gs_sampleid_to_rank_to_taxid_to_percentage:
                continue
            profile_sampleid_to_rank_to_taxid_to_percentage[label][sample_id] = load_data.get_rank_to_taxid_to_percentage(profile)

        profile_sample12_to_rank_to_braycurtis = {}
        # Compute Bray-Curtis for all combinations of samples
        for sample1, sample2 in sample_ids_combinations:
            if sample1 in profile_sampleid_to_rank_to_taxid_to_percentage[label] and sample2 in profile_sampleid_to_rank_to_taxid_to_percentage[label]:
                profile_sample12_to_rank_to_braycurtis[(sample1, sample2)] = bc.braycurtis(profile_sampleid_to_rank_to_taxid_to_percentage[label][sample1], profile_sampleid_to_rank_to_taxid_to_percentage[label][sample2])
        # Sort by rank
        profile_rank_to_sample12_to_braycurtis = defaultdict(dict)
        for sample12, rank_to_braycurtis in profile_sample12_to_rank_to_braycurtis.items():
            for rank, value in rank_to_braycurtis.items():
                profile_rank_to_sample12_to_braycurtis[rank][sample12] = value

        for rank in c.ALL_RANKS:
            gs_values = []
            profile_values = []
            if rank in gs_rank_to_sample12_to_braycurtis and rank in profile_rank_to_sample12_to_braycurtis:
                for (sample1, sample2), value in profile_rank_to_sample12_to_braycurtis[rank].items():
                    gs_values.append(gs_rank_to_sample12_to_braycurtis[rank][(sample1, sample2)])
                    profile_values.append(profile_rank_to_sample12_to_braycurtis[rank][(sample1, sample2)])
                plots_list += do_scatter_plot(profile_values, gs_values, output_dir, rank, label)
    return plots_list


def plot_shannon(rank_to_shannon_list, rank_to_shannon_gs, labels, output_dir, file_name):
    colors_list = create_colors_list()

    if rank_to_shannon_gs is None:
        # skip color of gold standard, if not present
        colors_list = colors_list[1:]
        ylabel = 'Shannon equitability (absolute diff. to gold standard)'
    else:
        labels = ['Gold standard'] + labels
        ylabel = 'Shannon equitability'

    fig, axs = plt.subplots(figsize=(6, 5))

    # force axis to be from 0 to 100%
    axs.set_ylim([0.0, 1.0])

    # turn on grid
    axs.grid(which='major', linestyle='-', linewidth='0.5', color='lightgrey')

    for i, rank_to_shannon in enumerate(rank_to_shannon_gs + rank_to_shannon_list if rank_to_shannon_gs is not None else rank_to_shannon_list):
        x = []
        y = []
        for j, rank in enumerate(c.ALL_RANKS, start=1):
            if rank in rank_to_shannon:
                x.append(j)
                y.append(rank_to_shannon[rank])
        axs.plot(x, y, color=colors_list[i], marker='o', markersize=10)

    axs.set_xticklabels([''] + c.ALL_RANKS)
    plt.setp(axs.get_xticklabels(), fontsize=9, rotation=25)

    axs.set_ylabel(ylabel)

    lgd = plt.legend(labels, bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0., handlelength=0, frameon=False)
    fig.savefig(os.path.join(output_dir, file_name + '.pdf'), dpi=100, format='pdf', bbox_extra_artists=(lgd,), bbox_inches='tight')
    fig.savefig(os.path.join(output_dir, file_name + '.png'), dpi=100, format='png', bbox_extra_artists=(lgd,), bbox_inches='tight')
    plt.close(fig)


def spider_plot(metrics, labels, rank_to_metric_to_toolvalues, output_dir, file_name, colors, grid_points=None, fill=False, absolute=False):
    N = len(labels)
    if N < 3:
        return []
    theta = spl.radar_factory(N, frame='polygon')
    fig, axes = plt.subplots(figsize=(9, 9), nrows=2, ncols=3, subplot_kw=dict(projection='radar'))
    fig.subplots_adjust(wspace=0.35, hspace=0.05, top=0.87, bottom=0.3)

    for ax, rank in zip(axes.flatten(), c.PHYLUM_SPECIES):
        if grid_points:
            ax.set_rgrids(grid_points, fontsize='xx-small')
        else:
            ax.set_rgrids([0.2, 0.4, 0.6, 0.8], ('', '', '', ''))  # get rid of the labels of the grid points
        ax.set_title(rank, weight='bold', size='medium', position=(0.5, 1.1),
                     horizontalalignment='center', verticalalignment='center')

        if absolute:
            metric_suffix = 'absolute'
        else:
            metric_suffix = ''
        # select only metrics in metrics list
        metrics_subdict = OrderedDict((metric, rank_to_metric_to_toolvalues[rank][metric + metric_suffix]) for metric in metrics)
        it = 1
        metric_to_toolindex = []
        for d, color in zip(metrics_subdict.values(), colors):
            # store index of tools without a value for the current metric
            metric_to_toolindex.append([i for i, x in enumerate(d) if x is None or np.isnan(x)])
            d = [0 if x is None or np.isnan(x) else x for x in d]

            ax.plot(theta, d, '--', color=color, linewidth=3, dashes=(it, 1))
            if fill:
                ax.fill(theta, d, facecolor=color, alpha=0.25)
            it += 1
        ax.set_varlabels(labels)

        ax.set_ylim([0.0, 1.0])
        ax.set_xlim([0.0, 1.0])

        # color red label of tools without a value for at least one metric
        xticklabels = ax.get_xticklabels()
        for metric in metric_to_toolindex:
            for toolindex in metric:
                xticklabels[toolindex].set_color([1, 0, 0])

    ax = axes[0, 0]
    ax.legend(metrics, loc=(1.6 - 0.353 * len(metrics), 1.3), labelspacing=0.1, fontsize='small', ncol=len(metrics))
    fig.savefig(os.path.join(output_dir, file_name + '.pdf'), dpi=100, format='pdf', bbox_inches='tight')
    fig.savefig(os.path.join(output_dir, file_name + '.png'), dpi=100, format='png', bbox_inches='tight')
    plt.close(fig)
    return [file_name]


def plot_braycurtis_l1norm(braycurtis_list, l1norm_list, labels, output_dir):
    colors_list = create_colors_list()
    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')

    reversed_ranks = c.ALL_RANKS.copy()
    reversed_ranks.reverse()
    rank_index = []

    color_index = 0
    for braycurtis, l1norm in zip(braycurtis_list, l1norm_list):
        for i, rank in enumerate(reversed_ranks, start=1):
            if rank in braycurtis and rank in l1norm:
                ax.scatter(braycurtis[rank], l1norm[rank], i, color=colors_list[color_index])
                rank_index.append(i)
        color_index += 1

    ax.set_xlabel('Bray-Curtis')
    ax.set_ylabel('L1 norm')

    rank_index = list(sorted(set(rank_index)))
    zlabels = [reversed_ranks[i - 1] for i in rank_index]
    ax.set_zticklabels(zlabels)

    legend_handles = []
    for i, label in enumerate(labels):
        line = mlines.Line2D([], [], color='white', markerfacecolor=colors_list[i], marker='o', markersize=8, label=label)
        legend_handles.append(line)

    plt.legend(handles=legend_handles, loc=2)

    fig.savefig(os.path.join(output_dir, 'braycurtis_l1norm.pdf'), dpi=100, format='pdf', bbox_inches='tight')
    fig.savefig(os.path.join(output_dir, 'braycurtis_l1norm.png'), dpi=100, format='png', bbox_inches='tight')
    plt.close(fig)


def plot_all(pd_metrics, labels, output_dir):
    metrics = [c.UNIFRAC, c.L1NORM, c.RECALL, c.PRECISION, c.FP]
    rank_to_metric_to_toolvalues = defaultdict(lambda : defaultdict(list))

    pd_copy = pd_metrics.copy()
    pd_copy.loc[pd_copy[pd.isnull(pd_copy['rank'])].index, 'rank'] = 'rank independent'
    # transform table
    pd_grouped = pd_copy.pivot_table(index=['rank', 'tool', 'sample'], columns='metric', values='value')

    # compute mean over samples and collapse rows
    cols = pd_grouped.columns
    pd_grouped = pd_grouped.groupby(['rank', 'tool'], sort=False)[cols].mean()

    # store maximum values per rank
    rank_to_max_fp = defaultdict()
    rank_to_max_unifrac = defaultdict()
    rank_to_max_l1norm = defaultdict()
    rank_to_max_recall = defaultdict()
    rank_to_max_precision = defaultdict()
    for rank in c.PHYLUM_SPECIES:
        pd_rank = pd_grouped.loc[(pd_grouped.index.get_level_values('rank') == rank) & (pd_grouped.index.get_level_values('tool') != c.GS)]
        rank_to_max_fp[rank] = pd_rank[c.FP].max()
        rank_to_max_l1norm[rank] = pd_rank[c.L1NORM].max()
        rank_to_max_recall[rank] = pd_rank[c.RECALL].max()
        rank_to_max_precision[rank] = pd_rank[c.PRECISION].max()
    pd_rank = pd_grouped.loc[(pd_grouped.index.get_level_values('rank') == 'rank independent') & (pd_grouped.index.get_level_values('tool') != c.GS)]
    rank_to_max_unifrac['rank independent'] = pd_rank[c.UNIFRAC].max()

    tool_to_rank_to_metric_to_value = defaultdict(lambda: defaultdict(dict))
    for (rank, tool), g in pd_grouped.groupby(['rank', 'tool']):
        if tool not in labels:
            continue
        if rank == 'rank independent':
            # relative values
            tool_to_rank_to_metric_to_value[tool][rank][c.UNIFRAC] = (g[c.UNIFRAC].values[0] / (rank_to_max_unifrac[rank]) if rank_to_max_unifrac[rank] > 0 else g[c.UNIFRAC].values[0])
        elif rank in c.PHYLUM_SPECIES:
            # absolute values
            tool_to_rank_to_metric_to_value[tool][rank][c.RECALL+'absolute'] = g[c.RECALL].values[0] if len(g[c.RECALL].values) > 0 else None
            tool_to_rank_to_metric_to_value[tool][rank][c.PRECISION+'absolute'] = g[c.PRECISION].values[0] if len(g[c.PRECISION].values) > 0 else None

            # relative values
            if rank_to_max_recall[rank] > 0:
                tool_to_rank_to_metric_to_value[tool][rank][c.RECALL] = (g[c.RECALL].values[0] / rank_to_max_recall[rank]) if len(g[c.RECALL].values) > 0 else None
            else:
                tool_to_rank_to_metric_to_value[tool][rank][c.RECALL] = g[c.RECALL].values[0] if len(g[c.RECALL].values) > 0 else None

            if rank_to_max_precision[rank] > 0:
                tool_to_rank_to_metric_to_value[tool][rank][c.PRECISION] = (g[c.PRECISION].values[0] / rank_to_max_precision[rank]) if len(g[c.PRECISION].values) > 0 else None
            else:
                tool_to_rank_to_metric_to_value[tool][rank][c.PRECISION] = g[c.PRECISION].values[0] if len(g[c.PRECISION].values) > 0 else None

            if rank_to_max_l1norm[rank] > 0:
                tool_to_rank_to_metric_to_value[tool][rank][c.L1NORM] = (g[c.L1NORM].values[0] / rank_to_max_l1norm[rank]) if len(g[c.L1NORM].values) > 0 else None
            else:
                tool_to_rank_to_metric_to_value[tool][rank][c.L1NORM] = g[c.L1NORM].values[0] if len(g[c.L1NORM].values) > 0 else None

            if rank_to_max_fp[rank] > 0:
                tool_to_rank_to_metric_to_value[tool][rank][c.FP] = (g[c.FP].values[0] / rank_to_max_fp[rank]) if len(g[c.FP].values) else None
            else:
                tool_to_rank_to_metric_to_value[tool][rank][c.FP] = g[c.FP].values[0] if len(g[c.FP].values) else None

    present_labels = []
    for label in labels:
        if label not in tool_to_rank_to_metric_to_value:
            continue
        else:
            present_labels.append(label)
        for rank in c.PHYLUM_SPECIES:
            for metric in metrics + [c.RECALL+'absolute', c.PRECISION+'absolute']:
                if metric in tool_to_rank_to_metric_to_value[label][rank]:
                    rank_to_metric_to_toolvalues[rank][metric].append(tool_to_rank_to_metric_to_value[label][rank][metric])
            rank_to_metric_to_toolvalues[rank][c.UNIFRAC].append(tool_to_rank_to_metric_to_value[label]['rank independent'][c.UNIFRAC])

    plots_list = spider_plot(metrics,
                             present_labels,
                             rank_to_metric_to_toolvalues,
                             output_dir,
                             'spider_plot',
                             ['b', 'g', 'r', 'k', 'm'])

    plots_list += spider_plot([c.RECALL, c.PRECISION],
                              present_labels,
                              rank_to_metric_to_toolvalues,
                              output_dir,
                              'spider_plot_recall_precision',
                              ['r', 'k'],
                              grid_points=[0.2, 0.4, 0.6, 0.8, 1.0],
                              fill=True,
                              absolute=True)

    # compute average shannon for gold standard
    pd_shannon_equit = pd_metrics[pd_metrics['metric'] == c.SHANNON_EQUIT]
    table1 = pd_shannon_equit[pd_shannon_equit['tool'] == c.GS][['rank', 'value']]
    rank_to_shannon_gs = table1.groupby('rank').mean().T.to_dict('records')

    tool_to_rank_to_shannon = defaultdict(dict)
    for (rank, tool), g in pd_grouped.groupby(['rank', 'tool']):
        if tool == c.GS or rank == 'rank independent':
            continue
        tool_to_rank_to_shannon[tool][rank] = g[c.SHANNON_EQUIT].values[0] if len(g[c.SHANNON_EQUIT].values) > 0 else None

    # compute differences between each tool and gold standard
    tool_to_rank_to_shannon_difference = defaultdict(dict)

    for tool in tool_to_rank_to_shannon.keys():
        for rank in tool_to_rank_to_shannon[tool].keys():
            if rank in rank_to_shannon_gs[0]:
                tool_to_rank_to_shannon_difference[tool][rank] = abs(rank_to_shannon_gs[0][rank] - tool_to_rank_to_shannon[tool][rank])

    # convert to list of dictionaries
    tool_to_rank_to_shannon = [tool_to_rank_to_shannon[label] for label in labels]
    tool_to_rank_to_shannon_difference = [tool_to_rank_to_shannon_difference[label] for label in labels]

    plot_shannon(tool_to_rank_to_shannon, rank_to_shannon_gs, labels, output_dir, 'plot_shannon')
    plot_shannon(tool_to_rank_to_shannon_difference, None, labels, output_dir, 'plot_shannon_diff')

    # pl.plot_braycurtis_l1norm(braycurtis_list, l1norm_list, labels, output_dir)
    return plots_list



GS = 'Gold standard'
SUPERKINGDOM = 'superkingdom'
PHYLUM = 'phylum'
CLASS = 'class'
ORDER = 'order'
FAMILY = 'family'
GENUS = 'genus'
SPECIES = 'species'
STRAIN = 'strain'
PHYLUM_SPECIES = [PHYLUM, CLASS, ORDER, FAMILY, GENUS, SPECIES]
ALL_RANKS = [SUPERKINGDOM, PHYLUM, CLASS, ORDER, FAMILY, GENUS, SPECIES, STRAIN]

UNIFRAC = 'Weighted Unifrac error'
UNW_UNIFRAC = 'Unweighted Unifrac error'
L1NORM = 'L1 norm error'
PRECISION = 'Purity (precision)'
RECALL = 'Completeness (recall)'
F1_SCORE = 'F1 score'
TP = "True positives"
FP = 'False positives'
FN = "False negatives"
OTUS = "Taxon counts"
JACCARD = "Jaccard index"
SHANNON_DIVERSITY = 'Shannon diversity'
SHANNON_EQUIT = 'Shannon equitability'
BRAY_CURTIS = 'Bray-Curtis distance'
ALL_METRICS = [UNIFRAC, UNW_UNIFRAC, L1NORM, PRECISION, RECALL, F1_SCORE, TP, FP, FN, JACCARD, SHANNON_DIVERSITY, SHANNON_EQUIT, BRAY_CURTIS]



#!/usr/bin/env python

import os
import sys
import errno
import argparse
import os.path
import l1norm as l1
import binary_metrics as bm
import unifrac_distance as uf
import shannon as sh
import braycurtis as bc
import rankings as rk
import html as html
import plots as pl
from utils import load_data
from utils import ProfilingTools as PF
from utils import constants as c
import pandas as pd
import numpy as np
from version import __version__


def make_sure_path_exists(path):
    try:
        os.makedirs(path)
    except OSError as exception:
        if exception.errno != errno.EEXIST:
            raise


def get_labels(labels, profiles_files):
    if labels:
        labels_list = [x.strip() for x in labels.split(',')]
        if len(labels_list) != len(profiles_files):
            exit('The number of labels does not match the number of files of profiles. Please check parameter -l, --labels.')
        return labels_list
    tool_id = []
    for profile_file in profiles_files:
        tool_id.append(profile_file.split('/')[-1])
    return tool_id


def compute_binary_metrics(query_profile, query_truth):
    all_metrics = bm.compute_tree_metrics(query_profile, query_truth)
    return all_metrics


def print_by_rank(output_dir, labels, pd_metrics):
    make_sure_path_exists(os.path.join(output_dir, "by_rank"))
    for rank in c.ALL_RANKS:
        # define ordering of rows, which is given my order of tool labels
        order_rows = labels
        # define ordering of columns, hard coded
        order_columns = [c.UNIFRAC, c.UNW_UNIFRAC, c.L1NORM, c.RECALL, c.PRECISION, c.F1_SCORE, c.TP, c.FP, c.FN, c.OTUS, c.JACCARD, c.SHANNON_DIVERSITY, c.SHANNON_EQUIT, c.BRAY_CURTIS]
        # subset to those information that either belong to the given rank or are rank independent, i.e. are unifrac values
        table = pd_metrics[(pd_metrics['rank'] == rank) | (pd_metrics['metric'].isin([c.UNIFRAC, c.UNW_UNIFRAC]))]
        # reformat the table with a pivot_table
        table = table.pivot_table(index=['tool', 'sample'], columns='metric', values='value')
        # select only tools in labels and get rid of gold standard
        table = table.loc[pd.IndexSlice[order_rows,:], order_columns]
        # define categorical column for ordering rows by tools
        table['tool_cat'] = pd.Categorical(table.index.get_level_values('tool'), categories=order_rows, ordered=True)
        # order table
        table = table.sort_values('tool_cat')
        table = table.loc[:, order_columns]
        # replace np.NaN with string "na" and write resulting table into a file
        table.fillna('na').to_csv(os.path.join(output_dir, "by_rank", rank + ".tsv"), sep='\t')


def print_by_tool(output_dir, pd_metrics):
    make_sure_path_exists(os.path.join(output_dir, "by_tool"))
    # define ordering of columns, hard coded
    order_columns = [c.UNIFRAC, c.UNW_UNIFRAC, c.L1NORM, c.RECALL, c.PRECISION, c.F1_SCORE, c.TP, c.FP, c.FN, c.OTUS, c.JACCARD, c.SHANNON_DIVERSITY, c.SHANNON_EQUIT, c.BRAY_CURTIS]
    for toolname, pd_metrics_tool in pd_metrics.groupby('tool'):
        if toolname == c.GS:
            continue
        table = pd_metrics_tool.pivot_table(index=['rank', 'sample'], columns='metric', values='value')
        # little hack to carry unifrac over to every rank
        for unifrac_col in order_columns[:2]:
            table[unifrac_col] = pd_metrics_tool[pd_metrics_tool['metric'] == unifrac_col]['value'].values[0]
        # order table
        table['rank_cat'] = pd.Categorical(table.index.get_level_values('rank'), categories=c.ALL_RANKS, ordered=True)
        table = table.sort_values('rank_cat')
        table = table.loc[:, order_columns]
        # replace np.NaN with string "na" and write resulting table into a file
        table.fillna('na').to_csv(os.path.join(output_dir, "by_tool", toolname + ".tsv"), sep='\t')


def compute_metrics(sample_metadata, profile, gs_pf_profile, gs_rank_to_taxid_to_percentage, rank_to_taxid_to_percentage):
    # Unifrac
    if isinstance(profile, PF.Profile):
        pf_profile = profile
    else:
        pf_profile = PF.Profile(sample_metadata=sample_metadata, profile=profile)
    unifrac = uf.compute_unifrac(gs_pf_profile, pf_profile)

    # Shannon
    shannon = sh.compute_shannon_index(rank_to_taxid_to_percentage)

    # L1 Norm
    l1norm = l1.compute_l1norm(gs_rank_to_taxid_to_percentage, rank_to_taxid_to_percentage)

    # Binary metrics
    binary_metrics = compute_binary_metrics(rank_to_taxid_to_percentage, gs_rank_to_taxid_to_percentage)

    # Bray-Curtis
    braycurtis = bc.braycurtis(gs_rank_to_taxid_to_percentage, rank_to_taxid_to_percentage)

    return unifrac, shannon, l1norm, binary_metrics, braycurtis


def load_profiles(gold_standard_file, profiles_files, no_normalization):
    normalize = False if no_normalization else True

    gs_samples_list = load_data.open_profile(gold_standard_file, normalize)
    sample_ids_list = []
    for sample in gs_samples_list:
        sample_id, sample_metadata, profile = sample
        sample_ids_list.append(sample_id)

    profiles_list_to_samples_list = []
    for profile_file in profiles_files:
        profiles_list_to_samples_list.append(load_data.open_profile(profile_file, normalize))

    return sample_ids_list, gs_samples_list, profiles_list_to_samples_list


def evaluate(gs_samples_list, profiles_list_to_samples_list, labels):
    gs_id_to_rank_to_taxid_to_percentage = {}
    gs_id_to_pf_profile = {}
    pd_metrics = pd.DataFrame()

    for sample in gs_samples_list:
        sample_id, sample_metadata, profile = sample
        gs_id_to_rank_to_taxid_to_percentage[sample_id] = load_data.get_rank_to_taxid_to_percentage(profile)
        gs_id_to_pf_profile[sample_id] = PF.Profile(sample_metadata=sample_metadata, profile=profile)
        unifrac, shannon, l1norm, binary_metrics, braycurtis = compute_metrics(sample_metadata,
                                                                               gs_id_to_pf_profile[sample_id],
                                                                               gs_id_to_pf_profile[sample_id],
                                                                               gs_id_to_rank_to_taxid_to_percentage[sample_id],
                                                                               gs_id_to_rank_to_taxid_to_percentage[sample_id])
        pd_metrics = pd.concat([pd_metrics, reformat_pandas(sample_id, c.GS, braycurtis, shannon, binary_metrics, l1norm, unifrac)], ignore_index=True)

    one_profile_assessed = False
    for samples_list, label in zip(profiles_list_to_samples_list, labels):
        for sample in samples_list:
            sample_id, sample_metadata, profile = sample

            # match the sample id of the gold standard and the predictions
            if sample_id in gs_id_to_rank_to_taxid_to_percentage:
                gs_rank_to_taxid_to_percentage = gs_id_to_rank_to_taxid_to_percentage[sample_id]
                gs_pf_profile = gs_id_to_pf_profile[sample_id]
            else:
                sys.stderr.write("Skipping assessment of {} for sample {}. Make sure the SampleID of the gold standard and the profile are identical.\n".format(label, sample_id))
                continue

            rank_to_taxid_to_percentage = load_data.get_rank_to_taxid_to_percentage(profile)

            unifrac, shannon, l1norm, binary_metrics, braycurtis = compute_metrics(sample_metadata, profile, gs_pf_profile,
                                                                                   gs_rank_to_taxid_to_percentage,
                                                                                   rank_to_taxid_to_percentage)
            pd_metrics = pd.concat([pd_metrics, reformat_pandas(sample_id, label, braycurtis, shannon, binary_metrics, l1norm, unifrac)], ignore_index=True)
            one_profile_assessed = True

    if not one_profile_assessed:
        sys.exit("No profile could be evaluated.")

    return pd_metrics


def reformat_pandas(sample_id, label, braycurtis, shannon, binary_metrics, l1norm, unifrac):
    """Reformats metrics data into one unified pandas DataFrame.
    
    Parameters
    ----------
    sample_id : str
    label : str
        str for tool name.
    braycurtis : float
    shannon : ?
    binary_metrics : ?
    l1norm : float
    unifrac : ?
    
    Returns
    -------
    Pandas.DataFrame with following columns: metric, rank, tool, value
    """
    # convert Unifrac
    pd_unifrac = pd.DataFrame(index=[sample_id], data=[unifrac], columns=[c.UNIFRAC, c.UNW_UNIFRAC]).stack().reset_index()
    pd_unifrac.columns = ['sample', 'metric', 'value']
    pd_unifrac['rank'] = np.nan
    pd_unifrac['tool'] = label

    # convert Shannon
    pd_shannon = pd.DataFrame([shannon[rank].get_pretty_dict() for rank in shannon.keys()]).set_index('rank').stack().reset_index().rename(columns={'level_1': 'metric', 0: 'value'})
    pd_shannon['metric'].replace(['diversity', 'equitability'], [c.SHANNON_DIVERSITY, c.SHANNON_EQUIT], inplace=True)
    pd_shannon['sample'] = sample_id
    pd_shannon['tool'] = label

    # convert L1 Norm
    pd_l1norm = pd.DataFrame(index=[sample_id], data=l1norm).stack().reset_index()
    pd_l1norm.columns = ['sample', 'rank', 'value']
    pd_l1norm['tool'] = label
    pd_l1norm['metric'] = c.L1NORM

    # convert Binary metrics
    pd_binary_metrics = pd.DataFrame([binary_metrics[rank].get_pretty_dict() for rank in binary_metrics.keys()]).set_index('rank').stack().reset_index().rename(columns={'level_1': 'metric', 0: 'value'})
    pd_binary_metrics['metric'].replace(['fp',
                                         'tp',
                                         "fn",
                                         "jaccard",
                                         "precision",
                                         "recall",
                                         "f1",
                                         "otus"],
                                        [c.FP,
                                         c.TP,
                                         c.FN,
                                         c.JACCARD,
                                         c.PRECISION,
                                         c.RECALL,
                                         c.F1_SCORE,
                                         c.OTUS], inplace=True)
    pd_binary_metrics['sample'] = sample_id
    pd_binary_metrics['tool'] = label

    # convert Bray-Curtis
    pd_braycurtis = pd.DataFrame(index=[sample_id], data=braycurtis).stack().reset_index()
    pd_braycurtis.columns = ['sample', 'rank', 'value']
    pd_braycurtis['tool'] = label
    pd_braycurtis['metric'] = c.BRAY_CURTIS

    return pd.concat([pd_braycurtis, pd_shannon, pd_binary_metrics, pd_l1norm, pd_unifrac], ignore_index=True)


def create_output_directories(output_dir, labels):
    make_sure_path_exists(output_dir)
    make_sure_path_exists(os.path.join(output_dir, 'gold_standard'))
    for label in labels:
        make_sure_path_exists(os.path.join(output_dir, "by_tool", label))


def main():
    parser = argparse.ArgumentParser(description="Compute all metrics for one or more taxonomic profiles")
    parser.add_argument("-g", "--gold_standard_file", help="Gold standard file", required=True)
    parser.add_argument("profiles_files", nargs='+', help="Files of profiles")
    parser.add_argument('-n', '--no_normalization', help="Do not normalize samples", action='store_true')
    parser.add_argument('-p', '--plot_abundances', help="Plot abundances in the gold standard (can take some minutes)", action='store_true')
    parser.add_argument('-l', '--labels', help="Comma-separated profiles names", required=False)
    parser.add_argument('-d', '--desc', help="Description for HTML page", required=False)
    parser.add_argument('-o', '--output_dir', help="Directory to write the results to", required=True)
    parser.add_argument('-v', '--version', action='version', version='%(prog)s ' + __version__)
    args = parser.parse_args()
    labels = get_labels(args.labels, args.profiles_files)
    output_dir = os.path.abspath(args.output_dir)

    sample_ids_list, gs_samples_list, profiles_list_to_samples_list = load_profiles(args.gold_standard_file,
                                                                                    args.profiles_files,
                                                                                    args.no_normalization)
    create_output_directories(output_dir, labels)
    plots_list = []
    if args.plot_abundances:
        plots_list += pl.plot_samples_hist(gs_samples_list, sample_ids_list, output_dir)

    pd_metrics = evaluate(gs_samples_list,
                          profiles_list_to_samples_list,
                          labels)

    pd_metrics[['tool', 'rank', 'metric', 'sample', 'value']].fillna('na').to_csv(os.path.join(output_dir, "results.tsv"), sep='\t', index=False)

    plots_list += pl.plot_beta_diversity(gs_samples_list, profiles_list_to_samples_list, sample_ids_list, labels, output_dir)

    plots_list += pl.plot_rarefaction_curves(gs_samples_list, output_dir)
    plots_list += pl.plot_rarefaction_curves(gs_samples_list, output_dir, log_scale=True)

    print_by_tool(output_dir, pd_metrics)
    print_by_rank(output_dir, labels, pd_metrics)
    plots_list += pl.plot_all(pd_metrics, labels, output_dir)

    pd_rankings = rk.highscore_table(pd_metrics)
    html.create_html(pd_rankings, pd_metrics, labels, sample_ids_list, plots_list, output_dir, args.desc)


if __name__ == "__main__":
    main()



import jinja2
import json
import numpy as np
import matplotlib.pyplot as plt

import mpld3
from mpld3 import plugins, utils


class HighlightLines(plugins.PluginBase):
    """A plugin to highlight lines on hover"""

    JAVASCRIPT = """
    mpld3.register_plugin("linehighlight", LineHighlightPlugin);
    LineHighlightPlugin.prototype = Object.create(mpld3.Plugin.prototype);
    LineHighlightPlugin.prototype.constructor = LineHighlightPlugin;
    LineHighlightPlugin.prototype.requiredProps = ["line_ids"];
    LineHighlightPlugin.prototype.defaultProps = {alpha_bg:0.3, alpha_fg:1.0}
    function LineHighlightPlugin(fig, props){
        mpld3.Plugin.call(this, fig, props);
    };

    LineHighlightPlugin.prototype.draw = function(){
      for(var i=0; i<this.props.line_ids.length; i++){
         var obj = mpld3.get_element(this.props.line_ids[i], this.fig),
             alpha_fg = this.props.alpha_fg;
             alpha_bg = this.props.alpha_bg;
         obj.elements()
             .on("mouseover", function(d, i){
                            d3.select(this).transition().duration(50)
                              .style("stroke-opacity", alpha_fg); })
             .on("mouseout", function(d, i){
                            d3.select(this).transition().duration(200)
                              .style("stroke-opacity", alpha_bg); });
      }
    };
    """

    def __init__(self, lines):
        self.lines = lines
        self.dict_ = {"type": "linehighlight",
                      "line_ids": [utils.get_id(line) for line in lines],
                      "alpha_bg": lines[0].get_alpha(),
                      "alpha_fg": 1.0}
N_paths = 50
N_steps = 100

x = np.linspace(0, 10, 100)
y = 0.1 * (np.random.random((N_paths, N_steps)) - 0.5)
y = y.cumsum(1)

fig, ax = plt.subplots()
lines = ax.plot(x, y.T, color='blue', lw=4, alpha=0.1)
plugins.connect(fig, HighlightLines(lines))
mpld3.save_html(fig, "fig.html")

#mpld3.show()



#!/usr/bin/env python

import sys
import argparse
import biom
from biom.util import biom_open
from collections import OrderedDict
from utils import load_data


def convert_to_biom(file_paths, output_file, is_json=False):
    tax_ids_row_mapping = OrderedDict()
    sample_ids = []
    sample_metadata = []
    data = {}
    observ_metadata = {}
    ids = []

    for file_path in file_paths:
        ids.append(file_path.split('/')[-1])
        try:
            samples_list = load_data.open_profile_from_tsv(file_path, False)
        except:
            sys.exit("Input file could not be read.")

        for i, sample in enumerate(samples_list):
            sample_id, header, profile = sample

            if sample_id == '':
                sample_id = 'S{}'.format(i)
                header['SAMPLEID'] = sample_id

            sample_ids.append(sample_id)
            sample_metadata.append(header)

            for prediction in profile:
                if prediction.taxid not in tax_ids_row_mapping:
                    tax_ids_row_mapping[prediction.taxid] = len(tax_ids_row_mapping)
                data[(tax_ids_row_mapping[prediction.taxid], i)] = prediction.percentage
                observ_metadata[prediction.taxid] = prediction.get_metadata()

    # sort metadata
    observ_metadata = [observ_metadata[key] for key in tax_ids_row_mapping.keys()]

    table = biom.Table(data=data,
                       observation_ids=list(tax_ids_row_mapping.keys()),
                       sample_ids=sample_ids,
                       observation_metadata=observ_metadata,
                       sample_metadata=sample_metadata,
                       table_id='_'.join(ids),
                       type='OTU table')

    if is_json:
        with open(output_file, 'w') as f:
            f.write(table.to_json(generated_by='OPAL'))
    else:
        with biom_open(output_file, 'w') as f:
            table.to_hdf5(f, 'OPAL')


def main():
    parser = argparse.ArgumentParser(description="Convert profile in CAMI Bioboxes format to BIOM format")
    parser.add_argument('files', nargs='+', help="Input file(s), one file per sample")
    parser.add_argument('-o', '--output_file', help="Output file", required=True)
    parser.add_argument('-j', '--json', action='store_true', help="Output in json (default: hdf5)")
    args = parser.parse_args()
    convert_to_biom(args.files, args.output_file, args.json)


if __name__ == "__main__":
    main()


import matplotlib.pyplot as plt
import numpy as np
import pandas as pd


# mu, sigma = 200, 25
# x = mu + sigma*np.random.randn(1000,3)
# x1 = mu + sigma*np.random.randn(990,1)
# x2 = mu + sigma*np.random.randn(980,1)
# x3 = mu + sigma*np.random.randn(1000,1)
#plt.figure()
#n, bins, patches = plt.hist(x, 30, stacked=True, normed = True)
x1 = [1,2,3]
x2 = [4,5,6]

df2 = pd.DataFrame(np.random.rand(10, 2), columns=['a', 'b']).T
# print(df2)
# print(df2.T)
#print(np.random.rand(10, 4))
#print(pd.DataFrame(np.random.rand(10, 4), columns=['a', 'b', 'c', 'd']))

#df2.T.plot(kind='bar', stacked=True, legend=False, colormap=plt.get_cmap('gist_rainbow'))

fig, axs = plt.subplots(figsize=(8, 5))
axs.tick_params(axis='x',
                which='both',
                bottom='off',
                top='off',
                labelbottom='off')
# axs.plot(df2.index, df2)
df2.plot(kind='bar', ax=axs, stacked=True, legend=False, colormap=plt.get_cmap('gist_rainbow'), width=.9)

#print(n)
#print(bins)

#plt.hist([x1,x2], histtype='barstacked')
plt.show()